# 23 "asllib/Lexer.mll"
 

open Tokens

open Error

module type CONFIG = sig
    (** Allow variables starting with a double underscore (__) *)
    val allow_double_underscore : bool
    val allow_unknown : bool
end

module Make (Config : CONFIG) = struct

exception LexerError

let new_line lexbuf = Lexing.new_line lexbuf; lexbuf
let bitvector_lit lxm = BITVECTOR_LIT (Bitvector.of_string lxm)
let mask_lit lxm = MASK_LIT (Bitvector.mask_of_string lxm)
let reserved_err s = Error.fatal_unknown_pos @@ (Error.ReservedIdentifier s)

let tr_name s = match s with
| "AND"           -> AND
| "array"         -> ARRAY
| "as"            -> AS
| "assert"        -> ASSERT
| "begin"         -> BEGIN
| "bit"           -> BIT
| "bits"          -> BITS
| "boolean"       -> BOOLEAN
| "case"          -> CASE
| "catch"         -> CATCH
| "config"        -> CONFIG
| "constant"      -> CONSTANT
| "__debug__"
| "__DEBUG__"     -> DEBUG
| "DIV"           -> DIV
| "DIVRM"         -> DIVRM
| "do"            -> DO
| "downto"        -> DOWNTO
| "else"          -> ELSE
| "elsif"         -> ELSIF
| "end"           -> END
| "enumeration"   -> ENUMERATION
| "XOR"           -> EOR
| "exception"     -> EXCEPTION
| "FALSE"         -> BOOL_LIT false
| "for"           -> FOR
| "func"          -> FUNC
| "getter"        -> GETTER
| "if"            -> IF
| "IN"            -> IN
| "integer"       -> INTEGER
| "let"           -> LET
| "looplimit"     -> LOOPLIMIT
| "MOD"           -> MOD
| "NOT"           -> NOT
| "of"            -> OF
| "OR"            -> OR
| "otherwise"     -> OTHERWISE
| "pass"          -> PASS
| "pragma"        -> PRAGMA
| "println"       -> PRINTLN
| "print"         -> PRINT
| "real"          -> REAL
| "record"        -> RECORD
| "recurselimit"  -> RECURSELIMIT
| "repeat"        -> REPEAT
| "return"        -> RETURN
| "setter"        -> SETTER
| "string"        -> STRING
| "subtypes"      -> SUBTYPES
| "then"          -> THEN
| "throw"         -> THROW
| "to"            -> TO
| "try"           -> TRY
| "TRUE"          -> BOOL_LIT true
| "type"          -> TYPE
| "UNKNOWN"       ->
    if Config.allow_unknown then ARBITRARY
    else Error.fatal_unknown_pos @@ (Error.ObsoleteSyntax s)
| "ARBITRARY"     -> ARBITRARY
| "Unreachable"   -> UNREACHABLE
| "until"         -> UNTIL
| "var"           -> VAR
| "when"          -> WHEN
| "where"         -> WHERE
| "while"         -> WHILE
| "with"          -> WITH
(* Reserved identifiers *)
| "SAMPLE" | "UNSTABLE"
| "_" | "any"
| "assume" | "assumes"
| "call" | "cast"
| "class" | "dict"
| "endcase" | "endcatch" | "endclass"
| "endevent" | "endfor" | "endfunc" | "endgetter"
| "endif" | "endmodule" | "endnamespace" | "endpackage"
| "endproperty" | "endrule" | "endsetter" | "endtemplate"
| "endtry" | "endwhile"
| "event" | "export"
| "extends" | "extern" | "feature"
| "gives"
| "iff" | "implies" | "import"
| "intersect" | "intrinsic"
| "invariant" | "list"
| "map" | "module" | "namespace" | "newevent"
| "newmap" | "original"
| "package" | "parallel"
| "port" | "private"
| "profile" | "property" | "protected" | "public"
| "requires" | "rethrow" | "rule"
| "shared" | "signal"
| "template"
| "typeof" | "union"
| "using"
| "ztype" -> reserved_err s
| x when not Config.allow_double_underscore
         && ASTUtils.string_starts_with ~prefix:"__" x -> reserved_err x
(* End of reserved identifiers *)
| x               -> IDENTIFIER x

# 125 "asllib/Lexer.ml"
let __ocaml_lex_tables = {
  Lexing.lex_base =
   "\000\000\251\255\252\255\253\255\254\255\255\255\001\000\002\000\
    \253\255\254\255\255\255\003\000\004\000\253\255\000\000\255\255\
    \086\000\206\255\196\000\000\000\210\255\216\255\000\000\001\000\
    \025\000\002\000\224\255\227\255\000\000\233\255\000\000\002\000\
    \236\255\004\000\004\000\000\000\002\000\005\000\244\255\006\000\
    \031\001\248\255\022\000\039\001\009\000\007\000\255\255\252\255\
    \031\000\032\000\063\001\118\001\049\001\034\001\247\255\107\001\
    \246\255\232\255\225\255\007\000\239\255\230\255\209\255\242\255\
    \241\255\028\000\240\255\237\255\235\255\217\255\221\255\228\255\
    \211\255\220\255\212\255\215\255\214\255\000\000\001\000\001\000\
    \001\000\000\000\005\000\001\000\005\000\208\255";
  Lexing.lex_backtrk =
   "\255\255\255\255\255\255\255\255\255\255\255\255\004\000\003\000\
    \255\255\255\255\255\255\004\000\003\000\255\255\001\000\255\255\
    \050\000\255\255\048\000\255\255\255\255\255\255\037\000\036\000\
    \255\255\032\000\255\255\255\255\026\000\255\255\021\000\029\000\
    \255\255\017\000\024\000\255\255\042\000\012\000\255\255\010\000\
    \255\255\255\255\004\000\004\000\033\000\001\000\255\255\255\255\
    \002\000\255\255\255\255\005\000\006\000\255\255\255\255\255\255\
    \255\255\255\255\255\255\255\255\255\255\255\255\255\255\255\255\
    \255\255\255\255\255\255\255\255\255\255\255\255\255\255\255\255\
    \255\255\255\255\255\255\255\255\255\255\255\255\255\255\255\255\
    \255\255\255\255\255\255\255\255\255\255\255\255";
  Lexing.lex_default =
   "\001\000\000\000\000\000\000\000\000\000\000\000\007\000\007\000\
    \000\000\000\000\000\000\012\000\012\000\000\000\255\255\000\000\
    \255\255\000\000\255\255\255\255\000\000\000\000\255\255\255\255\
    \255\255\255\255\000\000\000\000\255\255\000\000\255\255\255\255\
    \000\000\255\255\255\255\255\255\255\255\255\255\000\000\255\255\
    \255\255\000\000\255\255\255\255\255\255\255\255\000\000\000\000\
    \048\000\255\255\255\255\255\255\255\255\255\255\000\000\255\255\
    \000\000\000\000\000\000\255\255\000\000\000\000\000\000\000\000\
    \000\000\255\255\000\000\000\000\000\000\000\000\000\000\000\000\
    \000\000\000\000\000\000\000\000\000\000\255\255\255\255\255\255\
    \255\255\255\255\255\255\255\255\255\255\000\000";
  Lexing.lex_trans =
   "\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
    \000\000\000\000\000\000\008\000\255\255\013\000\255\255\000\000\
    \045\000\000\000\000\000\000\000\045\000\000\000\000\000\000\000\
    \000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
    \000\000\000\000\003\000\010\000\255\255\000\000\064\000\045\000\
    \000\000\255\255\000\000\000\000\075\000\014\000\255\255\015\000\
    \068\000\065\000\059\000\047\000\000\000\000\000\000\000\000\000\
    \048\000\000\000\076\000\074\000\072\000\070\000\069\000\062\000\
    \063\000\060\000\058\000\057\000\049\000\061\000\042\000\042\000\
    \042\000\042\000\042\000\042\000\042\000\042\000\042\000\042\000\
    \052\000\052\000\052\000\052\000\052\000\052\000\052\000\052\000\
    \052\000\052\000\066\000\071\000\002\000\009\000\255\255\045\000\
    \046\000\067\000\000\000\045\000\000\000\000\000\000\000\000\000\
    \000\000\082\000\084\000\000\000\077\000\081\000\005\000\078\000\
    \079\000\080\000\083\000\000\000\004\000\042\000\045\000\039\000\
    \041\000\085\000\000\000\000\000\035\000\040\000\027\000\032\000\
    \025\000\023\000\038\000\034\000\031\000\044\000\043\000\042\000\
    \042\000\042\000\042\000\042\000\042\000\042\000\042\000\042\000\
    \022\000\020\000\037\000\030\000\036\000\073\000\019\000\018\000\
    \018\000\018\000\018\000\018\000\018\000\018\000\018\000\018\000\
    \018\000\018\000\018\000\018\000\018\000\018\000\018\000\018\000\
    \018\000\018\000\018\000\018\000\018\000\018\000\018\000\018\000\
    \018\000\028\000\000\000\033\000\026\000\018\000\000\000\018\000\
    \018\000\018\000\018\000\018\000\018\000\018\000\018\000\018\000\
    \018\000\018\000\018\000\018\000\018\000\018\000\018\000\018\000\
    \018\000\018\000\018\000\018\000\018\000\018\000\018\000\018\000\
    \018\000\029\000\024\000\021\000\000\000\000\000\000\000\000\000\
    \000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
    \000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
    \000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
    \000\000\000\000\000\000\000\000\018\000\018\000\018\000\018\000\
    \018\000\018\000\018\000\018\000\018\000\018\000\000\000\000\000\
    \255\255\255\255\255\255\255\255\255\255\018\000\018\000\018\000\
    \018\000\018\000\018\000\018\000\018\000\018\000\018\000\018\000\
    \018\000\018\000\018\000\018\000\018\000\018\000\018\000\018\000\
    \018\000\018\000\018\000\018\000\018\000\018\000\018\000\255\255\
    \000\000\000\000\000\000\018\000\000\000\018\000\018\000\018\000\
    \018\000\018\000\018\000\018\000\018\000\018\000\018\000\018\000\
    \018\000\018\000\018\000\018\000\018\000\018\000\018\000\018\000\
    \018\000\018\000\018\000\018\000\018\000\018\000\018\000\040\000\
    \000\000\000\000\053\000\000\000\000\000\000\000\054\000\000\000\
    \000\000\056\000\000\000\000\000\000\000\000\000\000\000\040\000\
    \040\000\000\000\053\000\053\000\000\000\049\000\017\000\042\000\
    \042\000\042\000\042\000\042\000\042\000\042\000\042\000\042\000\
    \042\000\052\000\052\000\052\000\052\000\052\000\052\000\052\000\
    \052\000\052\000\052\000\000\000\000\000\000\000\000\000\051\000\
    \051\000\051\000\051\000\051\000\051\000\051\000\051\000\051\000\
    \051\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
    \051\000\051\000\051\000\051\000\051\000\051\000\042\000\000\000\
    \000\000\000\000\000\000\055\000\000\000\000\000\000\000\000\000\
    \052\000\000\000\054\000\000\000\000\000\000\000\000\000\053\000\
    \000\000\055\000\053\000\055\000\055\000\000\000\000\000\050\000\
    \051\000\051\000\051\000\051\000\051\000\051\000\051\000\051\000\
    \051\000\051\000\051\000\051\000\051\000\051\000\051\000\051\000\
    \000\000\000\000\000\000\000\000\000\000\000\000\000\000\051\000\
    \051\000\051\000\051\000\051\000\051\000\000\000\000\000\000\000\
    \000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
    \000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
    \000\000\000\000\000\000\000\000\000\000\051\000\000\000\051\000\
    \051\000\051\000\051\000\051\000\051\000\000\000\000\000\000\000\
    \000\000\000\000\000\000\000\000\000\000\055\000\000\000\000\000\
    \000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
    \000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
    \000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
    \000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
    \000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
    \000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
    \000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
    \000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
    \000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
    \000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
    \000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
    \000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
    \000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
    \000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
    \000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
    \000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
    \000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
    \000\000\000\000\000\000\000\000\000\000\000\000\000\000";
  Lexing.lex_check =
   "\255\255\255\255\255\255\255\255\255\255\255\255\255\255\255\255\
    \255\255\255\255\255\255\006\000\007\000\011\000\012\000\255\255\
    \045\000\255\255\255\255\255\255\045\000\255\255\255\255\255\255\
    \255\255\255\255\255\255\255\255\255\255\255\255\255\255\255\255\
    \255\255\255\255\000\000\006\000\007\000\255\255\035\000\045\000\
    \255\255\048\000\255\255\255\255\023\000\011\000\012\000\014\000\
    \031\000\034\000\037\000\044\000\255\255\255\255\255\255\255\255\
    \044\000\255\255\022\000\023\000\025\000\030\000\030\000\036\000\
    \036\000\037\000\037\000\039\000\042\000\059\000\042\000\042\000\
    \042\000\042\000\042\000\042\000\042\000\042\000\042\000\042\000\
    \049\000\049\000\049\000\049\000\049\000\049\000\049\000\049\000\
    \049\000\049\000\065\000\028\000\000\000\006\000\007\000\016\000\
    \016\000\033\000\255\255\016\000\255\255\255\255\255\255\255\255\
    \255\255\081\000\083\000\255\255\019\000\080\000\000\000\077\000\
    \078\000\079\000\082\000\255\255\000\000\042\000\016\000\016\000\
    \016\000\084\000\255\255\255\255\016\000\016\000\016\000\016\000\
    \016\000\016\000\016\000\016\000\016\000\016\000\016\000\016\000\
    \016\000\016\000\016\000\016\000\016\000\016\000\016\000\016\000\
    \016\000\016\000\016\000\016\000\016\000\024\000\016\000\016\000\
    \016\000\016\000\016\000\016\000\016\000\016\000\016\000\016\000\
    \016\000\016\000\016\000\016\000\016\000\016\000\016\000\016\000\
    \016\000\016\000\016\000\016\000\016\000\016\000\016\000\016\000\
    \016\000\016\000\255\255\016\000\016\000\016\000\255\255\016\000\
    \016\000\016\000\016\000\016\000\016\000\016\000\016\000\016\000\
    \016\000\016\000\016\000\016\000\016\000\016\000\016\000\016\000\
    \016\000\016\000\016\000\016\000\016\000\016\000\016\000\016\000\
    \016\000\016\000\016\000\016\000\255\255\255\255\255\255\255\255\
    \255\255\255\255\255\255\255\255\255\255\255\255\255\255\255\255\
    \255\255\255\255\255\255\255\255\255\255\255\255\255\255\255\255\
    \255\255\255\255\255\255\255\255\255\255\255\255\255\255\255\255\
    \255\255\255\255\255\255\255\255\018\000\018\000\018\000\018\000\
    \018\000\018\000\018\000\018\000\018\000\018\000\255\255\255\255\
    \000\000\006\000\007\000\011\000\012\000\018\000\018\000\018\000\
    \018\000\018\000\018\000\018\000\018\000\018\000\018\000\018\000\
    \018\000\018\000\018\000\018\000\018\000\018\000\018\000\018\000\
    \018\000\018\000\018\000\018\000\018\000\018\000\018\000\048\000\
    \255\255\255\255\255\255\018\000\255\255\018\000\018\000\018\000\
    \018\000\018\000\018\000\018\000\018\000\018\000\018\000\018\000\
    \018\000\018\000\018\000\018\000\018\000\018\000\018\000\018\000\
    \018\000\018\000\018\000\018\000\018\000\018\000\018\000\040\000\
    \255\255\255\255\053\000\255\255\255\255\255\255\040\000\255\255\
    \255\255\053\000\255\255\255\255\255\255\255\255\255\255\040\000\
    \040\000\255\255\053\000\053\000\255\255\043\000\016\000\043\000\
    \043\000\043\000\043\000\043\000\043\000\043\000\043\000\043\000\
    \043\000\052\000\052\000\052\000\052\000\052\000\052\000\052\000\
    \052\000\052\000\052\000\255\255\255\255\255\255\255\255\050\000\
    \050\000\050\000\050\000\050\000\050\000\050\000\050\000\050\000\
    \050\000\255\255\255\255\255\255\255\255\255\255\255\255\255\255\
    \050\000\050\000\050\000\050\000\050\000\050\000\043\000\255\255\
    \255\255\255\255\255\255\055\000\255\255\255\255\255\255\255\255\
    \052\000\255\255\055\000\255\255\255\255\255\255\255\255\040\000\
    \255\255\040\000\053\000\055\000\055\000\255\255\255\255\043\000\
    \050\000\050\000\050\000\050\000\050\000\050\000\051\000\051\000\
    \051\000\051\000\051\000\051\000\051\000\051\000\051\000\051\000\
    \255\255\255\255\255\255\255\255\255\255\255\255\255\255\051\000\
    \051\000\051\000\051\000\051\000\051\000\255\255\255\255\255\255\
    \255\255\255\255\255\255\255\255\255\255\255\255\255\255\255\255\
    \255\255\255\255\255\255\255\255\255\255\255\255\255\255\255\255\
    \255\255\255\255\255\255\255\255\255\255\051\000\255\255\051\000\
    \051\000\051\000\051\000\051\000\051\000\255\255\255\255\255\255\
    \255\255\255\255\255\255\255\255\255\255\055\000\255\255\255\255\
    \255\255\255\255\255\255\255\255\255\255\255\255\255\255\255\255\
    \255\255\255\255\255\255\255\255\255\255\255\255\255\255\255\255\
    \255\255\255\255\255\255\255\255\255\255\255\255\255\255\255\255\
    \255\255\255\255\255\255\255\255\255\255\255\255\255\255\255\255\
    \255\255\255\255\255\255\255\255\255\255\255\255\255\255\255\255\
    \255\255\255\255\255\255\255\255\255\255\255\255\255\255\255\255\
    \255\255\255\255\255\255\255\255\255\255\255\255\255\255\255\255\
    \255\255\255\255\255\255\255\255\255\255\255\255\255\255\255\255\
    \255\255\255\255\255\255\255\255\255\255\255\255\255\255\255\255\
    \255\255\255\255\255\255\255\255\255\255\255\255\255\255\255\255\
    \255\255\255\255\255\255\255\255\255\255\255\255\255\255\255\255\
    \255\255\255\255\255\255\255\255\255\255\255\255\255\255\255\255\
    \255\255\255\255\255\255\255\255\255\255\255\255\255\255\255\255\
    \255\255\255\255\255\255\255\255\255\255\255\255\255\255\255\255\
    \255\255\255\255\255\255\255\255\255\255\255\255\255\255\255\255\
    \255\255\255\255\255\255\255\255\255\255\255\255\255\255\255\255\
    \255\255\255\255\255\255\255\255\255\255\255\255\255\255\255\255\
    \255\255\255\255\255\255\255\255\255\255\255\255\255\255";
  Lexing.lex_base_code =
   "";
  Lexing.lex_backtrk_code =
   "";
  Lexing.lex_default_code =
   "";
  Lexing.lex_trans_code =
   "";
  Lexing.lex_check_code =
   "";
  Lexing.lex_code =
   "";
}

let rec escaped_string_chars acc lexbuf =
   __ocaml_lex_escaped_string_chars_rec acc lexbuf 0
and __ocaml_lex_escaped_string_chars_rec acc lexbuf __ocaml_lex_state =
  match Lexing.engine __ocaml_lex_tables __ocaml_lex_state lexbuf with
      | 0 ->
# 178 "asllib/Lexer.mll"
         ( Buffer.add_char acc '\n'; string_lit acc lexbuf )
# 344 "asllib/Lexer.ml"

  | 1 ->
# 179 "asllib/Lexer.mll"
         ( Buffer.add_char acc '\t'; string_lit acc lexbuf )
# 349 "asllib/Lexer.ml"

  | 2 ->
# 180 "asllib/Lexer.mll"
         ( Buffer.add_char acc '"'; string_lit acc lexbuf )
# 354 "asllib/Lexer.ml"

  | 3 ->
# 181 "asllib/Lexer.mll"
         ( Buffer.add_char acc '\\'; string_lit acc lexbuf )
# 359 "asllib/Lexer.ml"

  | 4 ->
# 182 "asllib/Lexer.mll"
                         ( raise LexerError )
# 364 "asllib/Lexer.ml"

  | __ocaml_lex_state -> lexbuf.Lexing.refill_buff lexbuf;
      __ocaml_lex_escaped_string_chars_rec acc lexbuf __ocaml_lex_state

and string_lit acc lexbuf =
   __ocaml_lex_string_lit_rec acc lexbuf 6
and __ocaml_lex_string_lit_rec acc lexbuf __ocaml_lex_state =
  match Lexing.engine __ocaml_lex_tables __ocaml_lex_state lexbuf with
      | 0 ->
# 185 "asllib/Lexer.mll"
          ( STRING_LIT (Buffer.contents acc) )
# 376 "asllib/Lexer.ml"

  | 1 ->
# 186 "asllib/Lexer.mll"
          ( escaped_string_chars acc lexbuf )
# 381 "asllib/Lexer.ml"

  | 2 ->
# 187 "asllib/Lexer.mll"
          ( Buffer.add_char acc '\n'; new_line lexbuf |> string_lit acc )
# 386 "asllib/Lexer.ml"

  | 3 ->
let
# 188 "asllib/Lexer.mll"
                          lxm
# 392 "asllib/Lexer.ml"
= Lexing.sub_lexeme lexbuf lexbuf.Lexing.lex_start_pos lexbuf.Lexing.lex_curr_pos in
# 188 "asllib/Lexer.mll"
                              ( Buffer.add_string acc lxm; string_lit acc lexbuf )
# 396 "asllib/Lexer.ml"

  | 4 ->
# 189 "asllib/Lexer.mll"
          ( raise LexerError )
# 401 "asllib/Lexer.ml"

  | __ocaml_lex_state -> lexbuf.Lexing.refill_buff lexbuf;
      __ocaml_lex_string_lit_rec acc lexbuf __ocaml_lex_state

and c_comments lexbuf =
   __ocaml_lex_c_comments_rec lexbuf 11
and __ocaml_lex_c_comments_rec lexbuf __ocaml_lex_state =
  match Lexing.engine __ocaml_lex_tables __ocaml_lex_state lexbuf with
      | 0 ->
# 197 "asllib/Lexer.mll"
                  ( token      lexbuf )
# 413 "asllib/Lexer.ml"

  | 1 ->
# 198 "asllib/Lexer.mll"
                  ( c_comments lexbuf )
# 418 "asllib/Lexer.ml"

  | 2 ->
# 199 "asllib/Lexer.mll"
                  ( new_line lexbuf |> c_comments )
# 423 "asllib/Lexer.ml"

  | 3 ->
# 200 "asllib/Lexer.mll"
                  ( c_comments lexbuf )
# 428 "asllib/Lexer.ml"

  | 4 ->
# 201 "asllib/Lexer.mll"
                  ( raise LexerError  )
# 433 "asllib/Lexer.ml"

  | __ocaml_lex_state -> lexbuf.Lexing.refill_buff lexbuf;
      __ocaml_lex_c_comments_rec lexbuf __ocaml_lex_state

and token lexbuf =
   __ocaml_lex_token_rec lexbuf 16
and __ocaml_lex_token_rec lexbuf __ocaml_lex_state =
  match Lexing.engine __ocaml_lex_tables __ocaml_lex_state lexbuf with
      | 0 ->
# 209 "asllib/Lexer.mll"
                               ( new_line lexbuf |> token         )
# 445 "asllib/Lexer.ml"

  | 1 ->
# 210 "asllib/Lexer.mll"
                               ( token lexbuf                     )
# 450 "asllib/Lexer.ml"

  | 2 ->
# 211 "asllib/Lexer.mll"
                               ( token lexbuf                     )
# 455 "asllib/Lexer.ml"

  | 3 ->
# 212 "asllib/Lexer.mll"
                               ( c_comments lexbuf                )
# 460 "asllib/Lexer.ml"

  | 4 ->
let
# 213 "asllib/Lexer.mll"
                 lxm
# 466 "asllib/Lexer.ml"
= Lexing.sub_lexeme lexbuf lexbuf.Lexing.lex_start_pos lexbuf.Lexing.lex_curr_pos in
# 213 "asllib/Lexer.mll"
                               ( INT_LIT(Z.of_string lxm)         )
# 470 "asllib/Lexer.ml"

  | 5 ->
let
# 214 "asllib/Lexer.mll"
                 lxm
# 476 "asllib/Lexer.ml"
= Lexing.sub_lexeme lexbuf lexbuf.Lexing.lex_start_pos lexbuf.Lexing.lex_curr_pos in
# 214 "asllib/Lexer.mll"
                               ( INT_LIT(Z.of_string lxm)         )
# 480 "asllib/Lexer.ml"

  | 6 ->
let
# 215 "asllib/Lexer.mll"
                  lxm
# 486 "asllib/Lexer.ml"
= Lexing.sub_lexeme lexbuf lexbuf.Lexing.lex_start_pos lexbuf.Lexing.lex_curr_pos in
# 215 "asllib/Lexer.mll"
                               ( REAL_LIT(Q.of_string lxm)        )
# 490 "asllib/Lexer.ml"

  | 7 ->
# 216 "asllib/Lexer.mll"
                               ( string_lit (Buffer.create 16) lexbuf )
# 495 "asllib/Lexer.ml"

  | 8 ->
let
# 217 "asllib/Lexer.mll"
                    lxm
# 501 "asllib/Lexer.ml"
= Lexing.sub_lexeme lexbuf (lexbuf.Lexing.lex_start_pos + 1) (lexbuf.Lexing.lex_curr_pos + -1) in
# 217 "asllib/Lexer.mll"
                               ( bitvector_lit lxm                )
# 505 "asllib/Lexer.ml"

  | 9 ->
let
# 218 "asllib/Lexer.mll"
                    lxm
# 511 "asllib/Lexer.ml"
= Lexing.sub_lexeme lexbuf (lexbuf.Lexing.lex_start_pos + 1) (lexbuf.Lexing.lex_curr_pos + -1) in
# 218 "asllib/Lexer.mll"
                               ( mask_lit lxm                     )
# 515 "asllib/Lexer.ml"

  | 10 ->
# 219 "asllib/Lexer.mll"
                               ( BNOT                             )
# 520 "asllib/Lexer.ml"

  | 11 ->
# 220 "asllib/Lexer.mll"
                               ( COMMA                            )
# 525 "asllib/Lexer.ml"

  | 12 ->
# 221 "asllib/Lexer.mll"
                               ( LT                               )
# 530 "asllib/Lexer.ml"

  | 13 ->
# 222 "asllib/Lexer.mll"
                               ( SHR                              )
# 535 "asllib/Lexer.ml"

  | 14 ->
# 223 "asllib/Lexer.mll"
                               ( BAND                             )
# 540 "asllib/Lexer.ml"

  | 15 ->
# 224 "asllib/Lexer.mll"
                               ( IMPL                             )
# 545 "asllib/Lexer.ml"

  | 16 ->
# 225 "asllib/Lexer.mll"
                               ( SHL                              )
# 550 "asllib/Lexer.ml"

  | 17 ->
# 226 "asllib/Lexer.mll"
                               ( RBRACKET                         )
# 555 "asllib/Lexer.ml"

  | 18 ->
# 227 "asllib/Lexer.mll"
                               ( RRBRACKET                        )
# 560 "asllib/Lexer.ml"

  | 19 ->
# 228 "asllib/Lexer.mll"
                               ( RPAR                             )
# 565 "asllib/Lexer.ml"

  | 20 ->
# 229 "asllib/Lexer.mll"
                               ( SLICING                          )
# 570 "asllib/Lexer.ml"

  | 21 ->
# 230 "asllib/Lexer.mll"
                               ( EQ                               )
# 575 "asllib/Lexer.ml"

  | 22 ->
# 231 "asllib/Lexer.mll"
                               ( LBRACE                           )
# 580 "asllib/Lexer.ml"

  | 23 ->
# 232 "asllib/Lexer.mll"
                               ( NEQ                              )
# 585 "asllib/Lexer.ml"

  | 24 ->
# 233 "asllib/Lexer.mll"
                               ( MINUS                            )
# 590 "asllib/Lexer.ml"

  | 25 ->
# 234 "asllib/Lexer.mll"
                               ( BEQ                              )
# 595 "asllib/Lexer.ml"

  | 26 ->
# 235 "asllib/Lexer.mll"
                               ( LBRACKET                         )
# 600 "asllib/Lexer.ml"

  | 27 ->
# 236 "asllib/Lexer.mll"
                               ( LLBRACKET                        )
# 605 "asllib/Lexer.ml"

  | 28 ->
# 237 "asllib/Lexer.mll"
                               ( LPAR                             )
# 610 "asllib/Lexer.ml"

  | 29 ->
# 238 "asllib/Lexer.mll"
                               ( DOT                              )
# 615 "asllib/Lexer.ml"

  | 30 ->
# 239 "asllib/Lexer.mll"
                               ( LEQ                              )
# 620 "asllib/Lexer.ml"

  | 31 ->
# 240 "asllib/Lexer.mll"
                               ( POW                              )
# 625 "asllib/Lexer.ml"

  | 32 ->
# 241 "asllib/Lexer.mll"
                               ( MUL                              )
# 630 "asllib/Lexer.ml"

  | 33 ->
# 242 "asllib/Lexer.mll"
                               ( RDIV                             )
# 635 "asllib/Lexer.ml"

  | 34 ->
# 243 "asllib/Lexer.mll"
                               ( EQ_OP                            )
# 640 "asllib/Lexer.ml"

  | 35 ->
# 244 "asllib/Lexer.mll"
                               ( BOR                              )
# 645 "asllib/Lexer.ml"

  | 36 ->
# 245 "asllib/Lexer.mll"
                               ( PLUS                             )
# 650 "asllib/Lexer.ml"

  | 37 ->
# 246 "asllib/Lexer.mll"
                               ( COLON                            )
# 655 "asllib/Lexer.ml"

  | 38 ->
# 247 "asllib/Lexer.mll"
                               ( ARROW                            )
# 660 "asllib/Lexer.ml"

  | 39 ->
# 248 "asllib/Lexer.mll"
                               ( RBRACE                           )
# 665 "asllib/Lexer.ml"

  | 40 ->
# 249 "asllib/Lexer.mll"
                               ( CONCAT                           )
# 670 "asllib/Lexer.ml"

  | 41 ->
# 250 "asllib/Lexer.mll"
                               ( COLON_COLON                      )
# 675 "asllib/Lexer.ml"

  | 42 ->
# 251 "asllib/Lexer.mll"
                               ( GT                               )
# 680 "asllib/Lexer.ml"

  | 43 ->
# 252 "asllib/Lexer.mll"
                               ( PLUS_COLON                       )
# 685 "asllib/Lexer.ml"

  | 44 ->
# 253 "asllib/Lexer.mll"
                               ( STAR_COLON                       )
# 690 "asllib/Lexer.ml"

  | 45 ->
# 254 "asllib/Lexer.mll"
                               ( SEMI_COLON                       )
# 695 "asllib/Lexer.ml"

  | 46 ->
# 255 "asllib/Lexer.mll"
                               ( GEQ                              )
# 700 "asllib/Lexer.ml"

  | 47 ->
# 256 "asllib/Lexer.mll"
                               ( ARROBASE_LOOPLIMIT               )
# 705 "asllib/Lexer.ml"

  | 48 ->
let
# 257 "asllib/Lexer.mll"
                    lxm
# 711 "asllib/Lexer.ml"
= Lexing.sub_lexeme lexbuf lexbuf.Lexing.lex_start_pos lexbuf.Lexing.lex_curr_pos in
# 257 "asllib/Lexer.mll"
                               ( tr_name lxm                      )
# 715 "asllib/Lexer.ml"

  | 49 ->
# 258 "asllib/Lexer.mll"
                               ( EOF                              )
# 720 "asllib/Lexer.ml"

  | 50 ->
# 259 "asllib/Lexer.mll"
                               ( raise LexerError                 )
# 725 "asllib/Lexer.ml"

  | __ocaml_lex_state -> lexbuf.Lexing.refill_buff lexbuf;
      __ocaml_lex_token_rec lexbuf __ocaml_lex_state

;;

# 260 "asllib/Lexer.mll"
 
end

# 736 "asllib/Lexer.ml"
